@@ -50,7 +50,7 @@ def run_forecast_ensemble(
     df = df[[DATE_COLUMN_NAME, VALUE_COLUMN_NAME]]
 
     # Create Future Forecast Periods
-    start_date = pd.to_datetime(dataframe[DATE_COLUMN_NAME]).max() + relativedelta(days=1)
+    start_date = pd.to_datetime(dataframe[DATE_COLUMN_NAME]).max() + relativedelta(days=6)
     end_date = start_date + relativedelta(years=forecast_horizon_years)
     # Add additional period if lag is requested
     if prediction_lag != 0:
@@ -92,6 +92,7 @@ def run_forecast_ensemble(
             weight = 0
         elif model_name == "Naive":
             weight = 0
+            estimate_df = model_dict["forecast_dataframe"]
         model_dict["weight"] = weight
         ensemble.append(model_dict)
 
@@ -102,6 +103,11 @@ def run_forecast_ensemble(
         total_weight = 1
     return_dataframe = df.copy().set_index(DATE_COLUMN_NAME)
     for model in ensemble:
+        # Replace negative values with the Naive value
+        forecast_df = model["forecast_dataframe"]
+        # mask = forecast_df[f"forecast_values_{model['name']}"] <= 0
+        # forecast_df.loc[mask, f"forecast_values_{model['name']}"] = estimate_df[mask, f"forecast_values_Naive"]
+
         model["weight"] = model["weight"] / total_weight
         return_dataframe = pd.merge(
             left=return_dataframe,
@@ -118,7 +124,6 @@ def run_forecast_ensemble(
                 how="outer",
                 left_index=True,
                 right_index=True)
-            forecast_df = model["forecast_dataframe"]
             forecast_df[f"forecast_values_{model['name']}"] = forecast_df[f"forecast_values_{model['name']}"] * model["weight"]
             forecast_df.rename(columns={f"forecast_values_{model['name']}": f"weighted_forecast_values_{model['name']}"}, inplace=True)
             return_dataframe = pd.merge(
@@ -158,7 +163,7 @@ def run_forecast_ensemble(
     })
 
     # Create degraded accuracies for all models
-    training_years = (df[DATE_COLUMN_NAME].max() - df[DATE_COLUMN_NAME].min()).days / 365
+    training_years = round((df[DATE_COLUMN_NAME].max() - df[DATE_COLUMN_NAME].min()).days / 365)
     for model in ensemble:
         degraded_accuracies = {}
         degraded_fit = {}
@@ -169,7 +174,7 @@ def run_forecast_ensemble(
                 for yy in range(1, int(year) + 1):
                     if yy > (2 * training_years):
                         multiplier *= 0.5
-                    elif yy > training_years:
+                    elif yy > 1:
                         multiplier *= 0.95
             degraded_accuracies[year] = multiplier * model["consolidated_metrics"]["accuracy"]
             degraded_fit[year] = multiplier * model["consolidated_metrics"]["fit"]
