@@ -9,6 +9,7 @@ from fbprophet import Prophet
 from greykite.framework.templates.autogen.forecast_config import ForecastConfig, MetadataParam, ModelComponentsParam, EvaluationPeriodParam
 from greykite.framework.templates.forecaster import Forecaster
 from greykite.framework.templates.model_templates import ModelTemplateEnum
+from neuralprophet import NeuralProphet
 from xgboost import XGBRegressor
 from sklearn.ensemble import RandomForestRegressor
 from statsmodels.tsa.statespace.sarimax import SARIMAX
@@ -127,9 +128,10 @@ class FBProphet(BaseModel):
             dataframes with actual and predicted values
         """
         df = dataframe.rename(columns={DATE_COLUMN_NAME: "ds", VALUE_COLUMN_NAME: "y"})
+        df["y"] = np.log(df["y"])
 
-        model = Prophet(yearly_seasonality=20)
-        model.fit(df, iter=10000)
+        model = Prophet(yearly_seasonality=10)
+        model.fit(df, iter=1000)
 
         future = model.make_future_dataframe(periods=len(forecast_period_list), freq=frequency)
         forecast = model.predict(future)
@@ -139,7 +141,55 @@ class FBProphet(BaseModel):
         df_train = df_forecast[~df_forecast[VALUE_COLUMN_NAME].isnull()]
         df_forecast = df_forecast[df_forecast[VALUE_COLUMN_NAME].isnull()]
 
-        return df_train[VALUE_COLUMN_NAME], df_train["yhat"], df_forecast["yhat"]
+        actual_values = df_train[VALUE_COLUMN_NAME]
+        predicted_values = np.exp(df_train["yhat"])
+        forecasted_values = np.exp(df_forecast["yhat"])
+
+        return actual_values, predicted_values, forecasted_values
+
+
+class FBNeuralProphet(BaseModel):
+
+    def __init__(self):
+        """Initializes prophet model"""
+        super().__init__()
+        self.__name__ = "FBNeuralProphet"
+
+    def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
+        """Runs rolling prediction using the model to return actual and predicted values
+
+        Args:
+            dataframe: time series dataframe to evaluate the model on
+
+        Returns:
+            dataframes with actual and predicted values
+        """
+        df = dataframe.rename(columns={DATE_COLUMN_NAME: "ds", VALUE_COLUMN_NAME: "y"})
+
+        model = NeuralProphet(yearly_seasonality=10)
+        model.fit(df=df, freq=frequency)
+
+        future = model.make_future_dataframe(
+            df=df,
+            periods=len(forecast_period_list),
+            n_historic_predictions=len(df))
+        forecast = model.predict(future)
+        forecast = forecast.rename(columns={"ds": DATE_COLUMN_NAME}).set_index(DATE_COLUMN_NAME)
+
+        df_forecast = pd.merge(
+            left=dataframe.set_index(DATE_COLUMN_NAME),
+            right=forecast.drop(columns=["y"]),
+            how="outer",
+            left_index=True,
+            right_index=True)
+        df_train = df_forecast[~df_forecast[VALUE_COLUMN_NAME].isnull()]
+        df_forecast = df_forecast[df_forecast[VALUE_COLUMN_NAME].isnull()]
+
+        actual_values = df_train[VALUE_COLUMN_NAME]
+        predicted_values = df_train["yhat1"]
+        forecasted_values = df_forecast["yhat1"]
+
+        return actual_values, predicted_values, forecasted_values
 
 
 class Naive(BaseModel):
@@ -291,7 +341,7 @@ class XGBoost(BaseModel):
         for column in result_df.columns:
             mask = mask & (~result_df[column].isnull())
         dataframe = pd.merge(left=result_df, right=period_indexes, how="left", left_index=True, right_index=True)
-        dataframe = dataframe[mask]
+        dataframe = dataframe[mask].drop(columns=["period_number"])
 
         return dataframe
 
@@ -362,12 +412,18 @@ class XGBoost(BaseModel):
 
     def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str,
             forecast_period_list: pd.DataFrame) -> tuple:
-        dataframe = dataframe.copy()
-        dataframe.set_index("period", inplace=True)
-        dataframe = dataframe.asfreq(frequency)
+        original_dataframe = dataframe.copy()
+
+        # Prepare dataframe for backtest
+        backfill_dataframe = utils.create_backfill_time_series(original_dataframe, frequency)
+        backfill_dataframe = pd.concat([original_dataframe, backfill_dataframe]).sort_values(by="period")
+        backfill_dataframe = backfill_dataframe.set_index("period").asfreq(frequency)
+
+        # Parse original dataframe for usage later
+        dataframe = original_dataframe.set_index("period").asfreq(frequency)
 
         # Prepare dataframe
-        dataframe_backtest = self.create_time_series_features(dataframe, frequency, backtest=True)
+        dataframe_backtest = self.create_time_series_features(backfill_dataframe, frequency, backtest=True)
 
         predicted = []
         # Split into train and test
@@ -429,7 +485,7 @@ class RandomForest(BaseModel):
         for column in result_df.columns:
             mask = mask & (~result_df[column].isnull())
         dataframe = pd.merge(left=result_df, right=period_indexes, how="left", left_index=True, right_index=True)
-        dataframe = dataframe[mask]
+        dataframe = dataframe[mask].drop(columns=["period_number"])
 
         return dataframe
 
@@ -500,14 +556,18 @@ class RandomForest(BaseModel):
 
     def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str,
             forecast_period_list: pd.DataFrame) -> tuple:
-        dataframe = dataframe.copy()
-        dataframe.set_index("period", inplace=True)
-        dataframe = dataframe.asfreq(frequency)
+        original_dataframe = dataframe.copy()
+
+        # Prepare dataframe for backtest
+        backfill_dataframe = utils.create_backfill_time_series(original_dataframe, frequency)
+        backfill_dataframe = pd.concat([original_dataframe, backfill_dataframe]).sort_values(by="period")
+        backfill_dataframe = backfill_dataframe.set_index("period").asfreq(frequency)
+
+        # Parse original dataframe for usage later
+        dataframe = original_dataframe.set_index("period").asfreq(frequency)
 
         # Prepare dataframe
-        dataframe_backtest = self.create_time_series_features(dataframe, frequency, backtest=True)
-        print(dataframe_backtest.head(3).T)
-        print(dataframe_backtest.columns)
+        dataframe_backtest = self.create_time_series_features(backfill_dataframe, frequency, backtest=True)
 
         predicted = []
         # Split into train and test
@@ -547,24 +607,35 @@ class SARIMA(BaseModel):
         self.__name__ = "SARIMA"
 
     def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
-        dataframe = dataframe.copy()
-        # Prepare dataframe
-        dataframe.set_index("period", inplace=True)
-        dataframe = dataframe.asfreq(frequency)
+        original_dataframe = dataframe.copy()
+
+        # Prepare dataframe for backtest
+        backfill_dataframe = utils.create_backfill_time_series(original_dataframe, frequency)
+        backfill_dataframe = pd.concat([original_dataframe, backfill_dataframe]).sort_values(by="period")
+        dataframe_backtest = backfill_dataframe.set_index("period").asfreq(frequency)
+
+        # Parse original dataframe for usage later
+        dataframe = original_dataframe.set_index("period").asfreq(frequency)
+
         if frequency == "W-MON":
             m = 52.5
         elif frequency == "MS":
             m = 12
 
         # Split into train and test
-        train_mask = dataframe.index <= train_end
-        train = dataframe[train_mask]
-        test = dataframe[~train_mask]
+        # Check if enough data is provided to train on a full year
+        train_start = dataframe_backtest.index.min()
+        if (train_end - train_start).days < 365:
+            if (dataframe_backtest.index.max() - train_start).days > 540:
+                train_end = train_start + relativedelta(years=1)
+        train_mask = dataframe_backtest.index <= train_end
+        train = dataframe_backtest[train_mask]
+        test = dataframe_backtest[~train_mask]
 
         # Create step-forward validation
         predicted = pd.DataFrame()
         previous_index = 0
-        period_horizon = len(forecast_period_list)
+        period_horizon = int(len(forecast_period_list) / 10)
 
         if period_horizon <= len(test):
             start = period_horizon
@@ -577,7 +648,7 @@ class SARIMA(BaseModel):
         backtest_range = range(start, end, step)
         for ii in backtest_range:
             model = SARIMAX(train, order=(1, 1, 2), seasonal_order=(1, 1, 2, m))
-            results = model.fit(method='powell')
+            results = model.fit(method='powell', disp=False)
 
             # Identify subset of test to forecast against
             test_subset = test.iloc[previous_index:ii]
@@ -603,13 +674,15 @@ class SARIMA(BaseModel):
                 predictions = pd.DataFrame(index=test_subset.index,
                                            data=results.forecast(steps=len(test_subset)).values)
                 predicted = pd.concat([predicted, predictions])
+        predicted[0] = predicted[0].apply(lambda x: x if x > 0 else 1)
 
         # Create future forecast
         model = SARIMAX(dataframe, order=(1, 1, 2), seasonal_order=(1, 1, 2, m))
-        results = model.fit(method='powell')
+        results = model.fit(method='powell', disp=False)
         predictions = results.forecast(steps=len(forecast_period_list))
 
         forecast = pd.Series(index=forecast_period_list, data=predictions)
+        forecast = forecast.apply(lambda x: x if x > 0 else 1)
 
         return test[VALUE_COLUMN_NAME], predicted[0].values, forecast
 
